define(
  ['require', 'apres', 'jquery', 'bootstrap', 'underscore'],
  function(require, apres, $, bootstrap, _) {
    var DeactivateWidget = function(elem, params) {
      var elem = $(elem);
      var loading_template_src = $("#deactivate-loading");
      var main_template_src = $("#deactivate-project");
      var modal_template_src = $("#confirm-delete-project-template");
      var main_template = _.template(main_template_src.html());
      var modal_template = _.template(modal_template_src.html());
      var loading_template = _.template(loading_template_src.html());
      var results;


      function message(message, classes) {
        elem.find("div.alert").removeClass().addClass("alert " + classes).html(message).show();
      }

      function message_hide() {
        elem.find("div.alert").hide();
      }

      // Render the main template
      function render(results) {
        var el = main_template(results);
        elem.html(el);
        // Load this later because it needs to be under the main template
        $("#confirm-delete-project").html(modal_template());
      }

      // Fetch & render the repo status
      function load(url) {
        var el = loading_template();
        elem.html(el);
        $.ajax({
          url: "/api/repo",
          type: "GET",
          data: {url: url},
          dataType: "json",
          success: function (data, ts, xhr) {
            results = data.results;
            render(results[0]);
          },
          error: function(xhr, ts, e) {
            if (xhr && xhr.responseText) {
                var data = $.parseJSON(xhr.responseText);
                message("Error loading repo config: " + data.errors[0], "alert-error");
            } else {
                message("Error loading repo config: " + e, "alert-error");
            }
          }

        });
      }

      // Delete a project & go to dashboard
      function del(url) {
        $.ajax({
          url: "/api/repo",
          type: "DELETE",
          data: {url: url},
          success: function(data, ts, xhr) {
            message("Project removed.", "alert-success");
            window.location = '/';
          },
          error: function(xhr, ts, e) {
            if (xhr && xhr.responseText) {
                var data = $.parseJSON(xhr.responseText);
                message("Error deleting project: " + data.errors[0], "alert-error");
            } else {
                message("Error deleting project: " + e, "alert-error");
            }
          }
        });
      }

      // Modify activation state
      function modify(url, state) {
        var data = {url: url, active: state};
        $.ajax({
          url: "/api/repo",
          type: "POST",
          data: data,
          dataType: "json",
          success: function(data, ts, xhr) {
            message("Active set to: " + state, "alert-success");
            load(repo_url);
          },
          error: function(xhr, ts, e) {
            if (xhr && xhr.responseText) {
                var data = $.parseJSON(xhr.responseText);
                message("Error adding webhook: " + data.errors[0], "alert-error");
            } else {
                message("Error adding webhook: " + e, "alert-error");
            }
          }
        });
      }

      // Handy bindings
      this.events = {
        'click .btn-activate': function() {
          modify(repo_url, true);
        },
        'click .btn-deactivate': function() {
          modify(repo_url, false);
        },
        'click .btn-delete-project': function() {
          del(repo_url);
        }
      };

      // There is a global "repo" object which is generated by the server
      var repo_url = apres.controller().params.repo_url;
      load(repo_url);

    };

    return DeactivateWidget;
  }
);
